package com.m2pool.common.security.aspect;

import com.m2pool.common.security.annotation.OpenApiPermissions;
import com.m2pool.common.security.annotation.RequiresLogin;
import com.m2pool.common.security.annotation.RequiresPermissions;
import com.m2pool.common.security.annotation.RequiresRoles;
import com.m2pool.common.security.auth.ApiUtil;
import com.m2pool.common.security.auth.AuthUtil;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;

/**
 * @Description 基于 Spring Aop 的注解鉴权
 * @Date 2024/6/13 18:47
 * @Author dy
 */
@Aspect
@Component
public class PreAuthAspect {

    /**
     * 构建
     */
    public PreAuthAspect(){

    }

    /**
     * 定义AOP签名 (切入所有使用鉴权注解的方法)
     */
    public static final String POINTCUT_SIGN = " @annotation(com.m2pool.common.security.annotation.RequiresLogin) || "
            + "@annotation(com.m2pool.common.security.annotation.RequiresPermissions) || "
            + "@annotation(com.m2pool.common.security.annotation.RequiresRoles) || "
            + "@annotation(com.m2pool.common.security.annotation.OpenApiPermissions)";

    /**
     * 声明AOP签名
     */
    @Pointcut(POINTCUT_SIGN)
    public void pointcut()
    {
    }

    /**
     * 环绕切入
     *
     * @param joinPoint 切面对象
     * @return 底层方法执行后的返回值
     * @throws Throwable 底层方法抛出的异常
     */
    @Around("pointcut()")
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {

        // 注解鉴权
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        checkMethodAnnotation(signature.getMethod());
        try
        {
            // 执行原有逻辑
            Object obj = joinPoint.proceed();
            return obj;
        }
        catch (Throwable e)
        {
            throw e;
        }
    }

    /**
     * 对一个Method对象进行注解检查
     */
    public void checkMethodAnnotation(Method method)
    {
        //校验 @RequiresLogin 注解
        RequiresLogin requiresLogin = method.getAnnotation(RequiresLogin.class);
        if(requiresLogin != null){
            AuthUtil.checkLogin();
        }

        //校验 @RequiresRoles 注解
        RequiresRoles requiresRoles = method.getAnnotation(RequiresRoles.class);
        if(requiresRoles != null){
            AuthUtil.checkRole(requiresRoles);
        }

        //校验 @RequiresPermissions 注解
        RequiresPermissions requiresPermissions = method.getAnnotation(RequiresPermissions.class);
        if(requiresPermissions != null){
            AuthUtil.checkPermission(requiresPermissions);
        }

        //校验 @OpenApiPermissions 注解
        OpenApiPermissions openApiPermissions = method.getAnnotation(OpenApiPermissions.class);
        if(openApiPermissions != null){
            System.out.println("校验 @OpenApiPermissions 注解");
            ApiUtil.checkPermission(openApiPermissions);
        }
    }
}
